unit break_out;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, LCLType, ExtCtrls,
  StdCtrls, LCLProc, LCLIntf;

type

  { TRacket }

 TRacket = class(TCustomControl)
  private

  protected

  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure EraseBackground(DC: HDC); override;
   procedure Paint; override;
  published
 end;

type

 { TBall }

 TBall = class(TCustomControl)
  private

  protected

  public

   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure EraseBackground(DC: HDC); override;
   procedure Paint; override;
  published
 end;

type

  { TMainForm }

  TMainForm = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    StaticText1: TStaticText;
    Timer1: TTimer;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormPaint(Sender: TObject);
    procedure MoveTheBall;
    procedure Timer1Timer(Sender: TObject);
  private
    backgroundbmp : TBitmap;
    racket        : TRacket;
    moveright     : boolean;
    moveleft      : boolean;
    moveball      : boolean;
    ball          : TBall;
    FHorStep      : integer;
    FVerStep      : integer;
    i1,i2         : integer;
    procedure CheckLeave(var aHorStep, aVerStep: integer);
    function Direction: integer;
  protected

  public

  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TBall }

constructor TBall.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Width  := 20;
 Height := 20;
 DoubleBuffered:=true;
end;

destructor TBall.Destroy;
begin
  inherited Destroy;
end;

procedure TBall.EraseBackground(DC: HDC);
begin
  inherited EraseBackground(DC);
end;

procedure TBall.Paint;
var tmpbmp : TBitmap;
begin
 tmpbmp             := TBitmap.Create;
  try
   tmpbmp.SetSize(width,height);
   tmpbmp.Canvas.Brush.Color := clWhite;
   tmpbmp.Canvas.FillRect(0,0,width,height);
   tmpbmp.Canvas.Brush.Color := clBlack;
   tmpbmp.Canvas.Ellipse(0,0,width,height);

   Canvas.Draw(0,0,tmpbmp);
  finally
   tmpbmp.Free;
  end;


 inherited Paint;
end;

{ TRacket }

constructor TRacket.Create(AOwner: TComponent);
begin
 inherited Create(AOwner);
 Width  := 100;
 Height := 20;
 DoubleBuffered:=true;

end;

destructor TRacket.Destroy;
begin
 inherited Destroy;
end;

procedure TRacket.EraseBackground(DC: HDC);
begin
 inherited EraseBackground(DC);

end;

procedure TRacket.Paint;
var tmpbmp : TBitmap;
begin
 tmpbmp             := TBitmap.Create;
  try
   tmpbmp.SetSize(width,height);
   tmpbmp.Canvas.Brush.Color := clNavy;
   tmpbmp.Canvas.FillRect(0,0, Width, Height);

   Canvas.Draw(0,0,tmpbmp);
  finally
   tmpbmp.Free;
  end;


 inherited Paint;
end;



//XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
 Left := (Monitor.Width div 2) - (width div 2);
 Top  := 20;

 backgroundbmp                    := TBitmap.Create;
 backgroundbmp.SetSize(800,800);
 backgroundbmp.Canvas.Brush.Color := clWhite;
 backgroundbmp.Canvas.FillRect(0,0,backgroundbmp.Width,backgroundbmp.Height);

 racket        := TRacket.Create(self);
 racket.Parent := self;
 racket.Left   := (Width div 2) - (racket.width div 2);
 racket.Top    := Height - (50 + racket.Height);

 ball          := TBall.Create(self);
 ball.Parent   := self;
 ball.Left     := (Width div 2) - (ball.width div 2);
 ball.Top      := racket.Top - ball.Height;
 ball.Visible  := false;

 Randomize;
 FHorStep         := 1;
 FVerStep         := 1;
 i1 := -1;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
 backgroundbmp.Free;
 racket.Free;
 ball.Free;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;Shift: TShiftState);
begin
  if Key = VK_Space then
   begin
    if Timer1.Enabled then Timer1.Enabled:=false else Timer1.Enabled:=true;
    ball.Visible:= true;
    moveball := true;
   end;

  if Key = VK_LEFT then
  begin
   moveleft := true;
  end;

 if Key = VK_RIGHT then
  begin
   moveright := true;
  end;
end;


procedure TMainForm.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
 if Key = VK_LEFT  then moveleft  := false;
 if Key = VK_RIGHT then moveright := false;
end;

procedure TMainForm.FormPaint(Sender: TObject);
begin
 Canvas.Draw(0,Height - backgroundbmp.Height,backgroundbmp);

end;

function TMainForm.Direction: integer;
begin
 Result :=random(2)+1;
end;

procedure TMainForm.CheckLeave(var aHorStep,aVerStep :integer);
var aHotspot : TPoint;
begin
 if Ball.BoundsRect.Right  >= Width                         then aHorStep := Direction *-1;
 if Ball.BoundsRect.Left   <= 0                             then aHorStep := Direction * 1;
 if Ball.BoundsRect.Top    <= Height - backgroundbmp.Height then aVerStep := Direction *-1;

 //lose the ball
 if Ball.BoundsRect.Bottom >= Height                        then
  begin
   aVerStep := Direction * 1;
   dec(i2);
   Label2.Caption := 'Loses: '+inttostr(i2);
  end;

 //or touch the racket:
 aHotspot.X := Ball.Left + (Ball.Width div 2);
 aHotspot.Y := Ball.BoundsRect.Bottom;
 if PtInRect(racket.BoundsRect,aHotspot) then
  begin
   aVerStep := Direction * 1;
   inc(i1);
   Label1.Caption := 'Goals: '+inttostr(i1);
  end;
end;

procedure TMainForm.MoveTheBall;
begin
 CheckLeave(FHorStep,FVerStep);
 Ball.Left := Ball.left + FHorStep;
 Ball.Top  := Ball.top  - FVerStep;
end;

procedure TMainForm.Timer1Timer(Sender: TObject);
begin
 if moveleft then
  begin
   racket.left := racket.left -1;
   if racket.Left < 0 then racket.Left:=0;
  end;
 if moveright then
  begin
   racket.Left := racket.left + 1;
   if racket.Left > (width - racket.Width) then racket.left := width - racket.Width;
  end;
 MoveTheBall;
end;


end.

